@@ -40,6 +40,26 @@
 #'
 #' @export
 summarize_confounds <- function(eyeris) {
+  # handle binocular objects
+  if (is_binocular_object(eyeris)) {
+    # process left and right eyes independently
+    left_result <- summarize_confounds(eyeris$left)
+    right_result <- summarize_confounds(eyeris$right)
+
+    # return combined structure
+    list_out <- list(
+      left = left_result,
+      right = right_result,
+      original_file = eyeris$original_file,
+      raw_binocular_object = eyeris$raw_binocular_object
+    )
+
+    class(list_out) <- "eyeris"
+
+    return(list_out)
+  }
+
+  # regular eyeris object processing
   tryCatch(
     {
       check_data(eyeris, "summarize_confounds")
@@ -52,14 +72,13 @@ summarize_confounds <- function(eyeris) {
   available_blocks <- get_block_numbers(eyeris)
 
   if (is.null(available_blocks)) {
-    cli::cli_abort("eyeris no blocks found error.")
+    cli::cli_abort("[EXIT] eyeris no blocks found error.")
   }
 
   hz <- eyeris$info$sample.rate
   if (is.null(hz)) {
     cli::cli_abort(
-      paste0("Sampling rate (Hz) must be present in tracker",
-             "metadata (i.e., eyeris$info)")
+      paste0("[EXIT] Sampling rate (Hz) must be present in tracker", "metadata (i.e., eyeris$info)")
     )
   }
 
@@ -73,7 +92,9 @@ summarize_confounds <- function(eyeris) {
 
     for (step_name in pupil_steps) {
       stepwise_pupil <- block_pupil_data[[step_name]]
-      if (is.null(stepwise_pupil)) next
+      if (is.null(stepwise_pupil)) {
+        next
+      }
 
       confounds_list[[block_name]][[step_name]] <-
         get_confounds_for_step(
@@ -91,7 +112,7 @@ summarize_confounds <- function(eyeris) {
   epoch_names <- grep("^epoch_", names(eyeris), value = TRUE)
 
   if (length(epoch_names) > 0) {
-    eyeris <- calculate_epoched_confounds(eyeris, epoch_names, hz, verbose)
+    eyeris <- calculate_epoched_confounds(eyeris, epoch_names, hz, verbose = TRUE)
   }
 
   eyeris
@@ -119,11 +140,12 @@ summarize_confounds <- function(eyeris) {
 #'
 #' @keywords internal
 get_confounds_for_step <- function(
-    pupil_df,
-    pupil_vec,
-    screen_width,
-    screen_height,
-    hz) {
+  pupil_df,
+  pupil_vec,
+  screen_width,
+  screen_height,
+  hz
+) {
   if (!("is_blink" %in% names(pupil_df))) {
     pupil_df <- tag_blinks(pupil_df, pupil_vec)
   }
@@ -139,13 +161,17 @@ get_confounds_for_step <- function(
   total_time_ms <- (nrow(pupil_df) - 1) / hz * 1000
   is_invalid <- is.na(pupil_vec) | pupil_df$is_blink | pupil_df$is_offscreen
   gap_rle <-
-    if (!any(is.na(is_invalid))) rle(is_invalid) else list(lengths = 0,
-                                                           values = FALSE)
+    if (!any(is.na(is_invalid))) rle(is_invalid) else list(lengths = 0, values = FALSE)
   gap_lengths <- gap_rle$lengths[gap_rle$values]
   blink_rle <-
-    if (!any(is.na(pupil_df$is_blink))) rle(pupil_df$is_blink) else list(
-      lengths = 0, values = FALSE
-    )
+    if (!any(is.na(pupil_df$is_blink))) {
+      rle(pupil_df$is_blink)
+    } else {
+      list(
+        lengths = 0,
+        values = FALSE
+      )
+    }
   blink_durs <- blink_rle$lengths[blink_rle$values] / hz * 1000
   total_blink_time <- sum(pupil_df$is_blink) / hz * 1000
 
@@ -160,18 +186,12 @@ get_confounds_for_step <- function(
     n_invalid = sum(is_invalid),
     prop_invalid = mean(is_invalid),
     n_gaps = length(gap_lengths),
-    max_gap_n_samples =
-      if (length(gap_lengths)) max(gap_lengths) else 0,
-    max_gap_duration_ms =
-      if (length(gap_lengths)) max(gap_lengths) / hz * 1000 else 0,
-    min_gap_n_samples =
-      if (length(gap_lengths)) min(gap_lengths) else 0,
-    min_gap_duration_ms =
-      if (length(gap_lengths)) min(gap_lengths) / hz * 1000 else 0,
-    mean_gap_n_samples =
-      if (length(gap_lengths)) mean(gap_lengths) else 0,
-    mean_gap_duration_ms =
-      if (length(gap_lengths)) mean(gap_lengths) / hz * 1000 else 0,
+    max_gap_n_samples = if (length(gap_lengths)) max(gap_lengths) else 0,
+    max_gap_duration_ms = if (length(gap_lengths)) max(gap_lengths) / hz * 1000 else 0,
+    min_gap_n_samples = if (length(gap_lengths)) min(gap_lengths) else 0,
+    min_gap_duration_ms = if (length(gap_lengths)) min(gap_lengths) / hz * 1000 else 0,
+    mean_gap_n_samples = if (length(gap_lengths)) mean(gap_lengths) else 0,
+    mean_gap_duration_ms = if (length(gap_lengths)) mean(gap_lengths) / hz * 1000 else 0,
     screen_width = screen_width,
     screen_height = screen_height,
     gaze_x_var_px = var(pupil_df$eye_x, na.rm = TRUE),
@@ -180,8 +200,7 @@ get_confounds_for_step <- function(
       calc_euclidean_dist(pupil_df$eye_x, pupil_df$eye_y, cx, cy),
       na.rm = TRUE
     ),
-    mean_gaze_distance_from_center_norm =
-      mean(pupil_df$gaze_dist_from_center, na.rm = TRUE),
+    mean_gaze_distance_from_center_norm = mean(pupil_df$gaze_dist_from_center, na.rm = TRUE),
     prop_clipped = mean(pupil_vec %in% range(pupil_vec, na.rm = TRUE)),
     n_blinks = length(blink_durs),
     blink_rate_hz = length(blink_durs) / (total_time_ms / 1000),
@@ -270,8 +289,7 @@ normalize_gaze_coords <- function(pupil_df, screen_width, screen_height) {
 #'   - `is_offscreen`: Logical indicating if gaze is outside screen boundaries
 #'
 #' @keywords internal
-tag_gaze_coords <- function(pupil_df, screen_width, screen_height,
-                            overshoot_buffer = 0.05) {
+tag_gaze_coords <- function(pupil_df, screen_width, screen_height, overshoot_buffer = 0.05) {
   pupil_df <- normalize_gaze_coords(
     pupil_df = pupil_df,
     screen_width = screen_width,
@@ -289,7 +307,7 @@ tag_gaze_coords <- function(pupil_df, screen_width, screen_height,
 #'
 #' Exports each block's confounds data to a separate CSV file.
 #' Each file will contain all pupil steps (e.g., pupil_raw, pupil_clean)
-#' as columns.
+#' as rows, with confound metrics as columns.
 #'
 #' @param confounds_list A nested list structure containing confounds data
 #' @param output_dir The directory where CSV files will be saved
@@ -301,9 +319,31 @@ tag_gaze_coords <- function(pupil_df, screen_width, screen_height,
 #' @return Invisibly returns a vector of created file paths
 #'
 #' @keywords internal
-export_confounds_to_csv <- function(confounds_list,
-                                    output_dir, filename_prefix, verbose,
-                                    run_num = NULL) {
+export_confounds_to_csv <- function(confounds_list, output_dir, filename_prefix, verbose, run_num = NULL) {
+  # handle binocular objects
+  if (is_binocular_object(confounds_list)) {
+    # process left and right eyes independently with appropriate suffixes
+    left_files <- export_confounds_to_csv(
+      confounds_list$left,
+      file.path(output_dir, "left"),
+      function(block_name) paste0(filename_prefix, "_eye-L_", block_name),
+      verbose,
+      run_num
+    )
+
+    right_files <- export_confounds_to_csv(
+      confounds_list$right,
+      file.path(output_dir, "right"),
+      function(block_name) paste0(filename_prefix, "_eye-R_", block_name),
+      verbose,
+      run_num
+    )
+
+    # return combined file paths
+    return(c(left_files, right_files))
+  }
+
+  # regular confounds list processing
   if (!dir.exists(output_dir)) {
     dir.create(output_dir, recursive = TRUE)
   }
@@ -312,7 +352,9 @@ export_confounds_to_csv <- function(confounds_list,
 
   for (block_name in names(confounds_list)) {
     block_data <- confounds_list[[block_name]]
-    if (length(block_data) == 0) next
+    if (length(block_data) == 0) {
+      next
+    }
 
     all_steps <- names(block_data)
     base_names <- sub("^pupil_", "", all_steps)
@@ -322,7 +364,9 @@ export_confounds_to_csv <- function(confounds_list,
     for (i in seq_along(all_steps)) {
       full_step_name <- all_steps[i]
       step_data <- block_data[[full_step_name]]
-      if (is.null(step_data)) next
+      if (is.null(step_data)) {
+        next
+      }
 
       if (!is.data.frame(step_data)) {
         step_data <- as.data.frame(step_data)
@@ -372,8 +416,26 @@ export_confounds_to_csv <- function(confounds_list,
 #' @return An updated `eyeris` object with epoched confounds
 #'
 #' @keywords internal
-calculate_epoched_confounds <- function(eyeris, epoch_names, hz,
-                                        verbose = TRUE) {
+calculate_epoched_confounds <- function(eyeris, epoch_names, hz, verbose = TRUE) {
+  # handle binocular objects
+  if (is_binocular_object(eyeris)) {
+    # process left and right eyes independently
+    left_result <- calculate_epoched_confounds(eyeris$left, epoch_names, hz, verbose)
+    right_result <- calculate_epoched_confounds(eyeris$right, epoch_names, hz, verbose)
+
+    # return combined structure
+    list_out <- list(
+      left = left_result,
+      right = right_result,
+      original_file = eyeris$original_file
+    )
+
+    class(list_out) <- "eyeris"
+
+    return(list_out)
+  }
+
+  # regular eyeris object processing
   eyeris$confounds$epoched_timeseries <- list()
   eyeris$confounds$epoched_epoch_wide <- list()
 
@@ -382,13 +444,19 @@ calculate_epoched_confounds <- function(eyeris, epoch_names, hz,
     eyeris$confounds$epoched_epoch_wide[[epoch_name]] <- list()
 
     for (block_name in names(eyeris[[epoch_name]])) {
-      if (block_name == "info") next
+      if (block_name == "info") {
+        next
+      }
 
       epoch_data <- eyeris[[epoch_name]][[block_name]]
 
-      if (is.null(epoch_data) ||
-            !is.data.frame(epoch_data) ||
-            nrow(epoch_data) == 0) next
+      if (
+        is.null(epoch_data) ||
+          !is.data.frame(epoch_data) ||
+          nrow(epoch_data) == 0
+      ) {
+        next
+      }
 
       epoch_ids <- unique(epoch_data$matched_event)
 
@@ -397,11 +465,12 @@ calculate_epoched_confounds <- function(eyeris, epoch_names, hz,
       epoch_wide_confounds <- list()
       step_specific_confounds <- list()
 
-      # nolint start
       for (id in epoch_ids) {
         epoch_subset <- epoch_data[epoch_data$matched_event == id, ]
 
-        if (nrow(epoch_subset) == 0) next
+        if (nrow(epoch_subset) == 0) {
+          next
+        }
 
         epoch_duration_ms <- (nrow(epoch_subset) - 1) / hz * 1000
         block_blinks <- eyeris$blinks[[block_name]]
@@ -409,16 +478,16 @@ calculate_epoched_confounds <- function(eyeris, epoch_names, hz,
         if ("baseline_period" %in% names(epoch_subset)) {
           baseline_start <- min(epoch_subset$baseline_period)
           baseline_end <- max(epoch_subset$baseline_period)
-          n_blinks_baseline <- sum(sapply(seq_len(nrow(block_blinks)),
-                                          function(i) {
-            max(block_blinks$stime[i], baseline_start) <=
-              min(block_blinks$etime[i], baseline_end)
+          n_blinks_baseline <- sum(sapply(seq_len(nrow(block_blinks)), function(i) {
+            max(block_blinks$stime[i], baseline_start) <= min(block_blinks$etime[i], baseline_end)
           }))
         }
         first_blink_time <- NA
         epoch_start_time <- min(epoch_subset$time_orig)
-        blinks_in_epoch <- block_blinks[block_blinks$stime >= epoch_start_time &
-          block_blinks$stime <= max(epoch_subset$time_orig), ]
+        blinks_in_epoch <- block_blinks[
+          block_blinks$stime >= epoch_start_time &
+            block_blinks$stime <= max(epoch_subset$time_orig),
+        ]
         if (nrow(blinks_in_epoch) > 0) {
           first_blink_time <-
             (min(blinks_in_epoch$stime) - epoch_start_time) / hz * 1000
@@ -441,7 +510,9 @@ calculate_epoched_confounds <- function(eyeris, epoch_names, hz,
         for (step_name in pupil_steps) {
           pupil_vec <- epoch_subset[[step_name]]
 
-          if (all(is.na(pupil_vec))) next
+          if (all(is.na(pupil_vec))) {
+            next
+          }
 
           base_confounds <- get_confounds_for_step(
             pupil_df = epoch_subset,
@@ -453,8 +524,10 @@ calculate_epoched_confounds <- function(eyeris, epoch_names, hz,
 
           pre_epoch_window <- c(epoch_start_time - 200, epoch_start_time)
           pre_epoch_data <- eyeris$timeseries[[block_name]] |>
-            dplyr::filter(time_orig >= pre_epoch_window[1] &
-                            time_orig <= pre_epoch_window[2])
+            dplyr::filter(
+              time_orig >= pre_epoch_window[1] &
+                time_orig <= pre_epoch_window[2]
+            )
 
           step_df <- data.frame(
             matched_event = id,
@@ -493,4 +566,3 @@ calculate_epoched_confounds <- function(eyeris, epoch_names, hz,
 
   return(eyeris)
 }
-# nolint end
