@@ -56,14 +56,49 @@
 #'     call_info
 #'   }
 #'
-#'   pipeline_handler(
-#'     eyeris,
-#'     winsorize_pupil,
-#'     "winsorize",
-#'     lower = lower,
-#'     upper = upper,
-#'     call_info = call_info
-#'   )
+#'   # handle binocular objects
+#'   if (eyeris:::is_binocular_object(eyeris)) {
+#'     # process left and right eyes independently
+#'     left_result <- eyeris$left |>
+#'       pipeline_handler(
+#'         winsorize_pupil,
+#'         "winsorize",
+#'         lower = lower,
+#'         upper = upper,
+#'         call_info = call_info
+#'       )
+#'
+#'     right_result <- eyeris$right |>
+#'       pipeline_handler(
+#'         winsorize_pupil,
+#'         "winsorize",
+#'         lower = lower,
+#'         upper = upper,
+#'         call_info = call_info
+#'     )
+#'
+#'     # return combined structure
+#'     list_out <- list(
+#'       left = left_result,
+#'       right = right_result,
+#'       original_file = eyeris$original_file,
+#'       raw_binocular_object = eyeris$raw_binocular_object
+#'     )
+#'
+#'     class(list_out) <- "eyeris"
+#'
+#'     return(list_out)
+#'   } else {
+#'     # regular eyeris object, process normally
+#'     eyeris |>
+#'       pipeline_handler(
+#'         winsorize_pupil,
+#'         "winsorize",
+#'         lower = lower,
+#'         upper = upper,
+#'         call_info = call_info
+#'       )
+#'   }
 #' }
 #'
 #' # and voilà, you can now connect your custom extension
@@ -93,7 +128,9 @@ pipeline_handler <- function(eyeris, operation, new_suffix, ...) {
     call_info <- dots$call_info
     dots$call_info <- NULL # Remove call_info from dots
   }
-  if (!is.list(eyeris$params)) eyeris$params <- list()
+  if (!is.list(eyeris$params)) {
+    eyeris$params <- list()
+  }
   # ensure call_info is a list with call_stack and parameters
   if (!is.null(call_info) && !is.list(call_info)) {
     call_info <- list(call_stack = call_info, parameters = dots)
@@ -119,31 +156,38 @@ pipeline_handler <- function(eyeris, operation, new_suffix, ...) {
   if (is.list(eyeris$timeseries) && !is.data.frame(eyeris$timeseries)) {
     is_multiblock <- TRUE
   } else {
-    if (is.null(prev_operation) ||
-          length(prev_operation) == 0 ||
-          prev_operation == "") {
+    if (
+      is.null(prev_operation) ||
+        length(prev_operation) == 0 ||
+        prev_operation == ""
+    ) {
       cli::cli_abort(
         paste0(
-          "Latest pointer is empty or NULL.",
+          "[EXIT] Latest pointer is empty or NULL.",
           "This indicates a pipeline initialization error."
         )
       )
     }
     if (grepl("_([^_]+)_\\1", prev_operation)) {
       cli::cli_abort(paste(
-        "Corrupted latest pointer detected:", prev_operation,
+        "[EXIT] Corrupted latest pointer detected:",
+        prev_operation,
         "This indicates a pipeline error. Please restart the pipeline."
       ))
     }
     is_multiblock <- FALSE
   }
 
-  output_col <- paste0(prev_operation, "_", new_suffix)
-  if (grepl("_([^_]+)_\\1", output_col)) {
-    cli::cli_abort(paste(
-      "Attempting to create corrupted column name:", output_col,
-      "This indicates a pipeline processing error. Please check your data."
-    ))
+  # only create output_col and check for single-block data
+  if (!is_multiblock) {
+    output_col <- paste0(prev_operation, "_", new_suffix)
+    if (grepl("_([^_]+)_\\1", output_col)) {
+      cli::cli_abort(paste(
+        "[EXIT] Attempting to create corrupted column name:",
+        output_col,
+        "This indicates a pipeline processing error. Please check your data."
+      ))
+    }
   }
 
   if (is.list(eyeris$timeseries) && !is.data.frame(eyeris$timeseries)) {
@@ -164,33 +208,40 @@ pipeline_handler <- function(eyeris, operation, new_suffix, ...) {
       if (new_suffix != "epoch") {
         data <- eyeris$timeseries[[i_block]]
         block_prev_operation <- eyeris$latest[[i_block]]
-        if (is.null(block_prev_operation) ||
-              length(block_prev_operation) == 0 ||
-              block_prev_operation == "") {
+        if (
+          is.null(block_prev_operation) ||
+            length(block_prev_operation) == 0 ||
+            block_prev_operation == ""
+        ) {
           cli::cli_abort(paste(
-            "Latest pointer for block",
+            "[EXIT] Latest pointer for block",
             i_block,
             "is empty or NULL."
           ))
         }
         if (grepl("_([^_]+)_\\1", block_prev_operation)) {
           cli::cli_abort(paste(
-            "Corrupted latest pointer detected for block",
-            i_block, ":", block_prev_operation,
+            "[EXIT] Corrupted latest pointer detected for block",
+            i_block,
+            ":",
+            block_prev_operation,
             "This indicates a pipeline error. Please restart the pipeline."
           ))
         }
         block_output_col <- paste0(block_prev_operation, "_", new_suffix)
         if (grepl("_([^_]+)_\\1", block_output_col)) {
           cli::cli_abort(paste(
-            "Attempting to create corrupted column name for block",
-            i_block, ":", block_output_col,
+            "[EXIT] Attempting to create corrupted column name for block",
+            i_block,
+            ":",
+            block_output_col,
             "This indicates a pipeline error. Please check your data."
           ))
         }
         if (new_suffix == "detrend") {
           list_detrend <- do.call(
-            operation, c(list(data, block_prev_operation), dots)
+            operation,
+            c(list(data, block_prev_operation), dots)
           )
           data["detrend_fitted_values"] <- list_detrend$fitted_values
           data[[block_output_col]] <- list_detrend$residuals
@@ -200,7 +251,8 @@ pipeline_handler <- function(eyeris, operation, new_suffix, ...) {
           eyeris$detrend_coefs[[i_block]] <- list_detrend$coefficients
         } else if (new_suffix == "bin" || new_suffix == "downsample") {
           list_ds_bin <- do.call(
-            operation, c(list(data, block_prev_operation), dots)
+            operation,
+            c(list(data, block_prev_operation), dots)
           )
           data <- list_ds_bin$downsampled_df |>
             dplyr::select(
@@ -217,7 +269,8 @@ pipeline_handler <- function(eyeris, operation, new_suffix, ...) {
             )
         } else {
           data[[block_output_col]] <- do.call(
-            operation, c(list(data, block_prev_operation), dots)
+            operation,
+            c(list(data, block_prev_operation), dots)
           )
         }
         eyeris$timeseries[[i_block]] <- data
@@ -261,7 +314,8 @@ pipeline_handler <- function(eyeris, operation, new_suffix, ...) {
         data[[output_col]] <- list_detrend$residuals
       } else {
         data[[output_col]] <- do.call(
-          operation, c(list(data, prev_operation), dots)
+          operation,
+          c(list(data, prev_operation), dots)
         )
       }
       eyeris$timeseries <- data
